;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Horizontal Power Thwomp
;;
;; First Extra Bit: YES
;; If set, Thomp goes left instead of right. If not, it goes right.
;;
;; DISCLAIMER: This was based heavily off of Mikeyk's and MiOr's code. I just assembled it together.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPRITE_GRAVITY		dcb $FC,$04
RETURN_SPEED		dcb $10,$F0
DIRECTION_CHECK		dcb $00,$01
POS_TABLE		dcb $F6,$08
POS_TABLE_2		dcb $FF,$00

ANGRY_TILE		= $CA

X_OFFSET		dcb $FC,$04,$FC,$04,$00
Y_OFFSET		dcb $00,$00,$10,$10,$08
TILE_MAP		dcb $8E,$8E,$AE,$AE,$C8
PROPERTIES		dcb $03,$43,$03,$43,$03

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "INIT"
			LDA $E4,x
			ORA #$08
			STA $E4,x
			STA $151C,x
			LDA $7FAB10,x
			AND #$04
			BNE INIT_END
			INC $C2,x
INIT_END		RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "MAIN"
			PHB
			PHK
			PLB
			JSR SPRITE_CODE_START
			PLB
			RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN			RTS  

SPRITE_CODE_START	JSR SUB_GFX

			LDA $14C8,x
			CMP #$08
			BNE RETURN
			LDA $9D
			BNE RETURN

			STZ $AA,x

			JSR SUB_OFF_SCREEN_X0
			JSL $01A7DC

			LDA $1534,x
			BEQ WAITING
			DEC A
			BEQ ATTACKING
			JMP RETURNING

;-----------------------------------------------------------------------------------------
; state 0
;-----------------------------------------------------------------------------------------

WAITING			LDA $186C,x
			ORA $15A0,x
			BNE RETURN0

			STZ $1528,x
			JSR SUB_HORZ_POS
			TYA
			LDY $C2,x
			CMP DIRECTION_CHECK,y
			BEQ RETURN0

			LDA $0E
			CPY #$00
			BNE RIGHT_THWOMP
			SEC
			SBC #$40
			PHA
			CMP #$48
			BCC THWOMP_4
			LDA #$01
			STA $1528,x
THWOMP_4		PLA
			CMP #$60
			BCC RETURN0

NEXT_STATE		LDA #$02
			STA $1528,x

			INC $1534,x

			STZ $B6,x

RETURN0			RTS

RIGHT_THWOMP		CLC
			ADC #$40
			PHA
			CMP #$B8
			BCS THWOMP_4_2
			LDA #$01
			STA $1528,x
THWOMP_4_2		PLA
			CMP #$A0
			BCC NEXT_STATE
			RTS

;-----------------------------------------------------------------------------------------
; state 1
;-----------------------------------------------------------------------------------------

ATTACKING		JSL $01802A

			LDY $C2,x
			LDA $B6,x
			CMP #$C0
			BMI DONT_INC_SPEED
			CLC
			ADC SPRITE_GRAVITY,y
			STA $B6,x

DONT_INC_SPEED		LDA $E4,x
			PHA
			CLC
			ADC POS_TABLE,y
			STA $E4,x
			LDA $14E0,x
			PHA
			ADC POS_TABLE_2,y
			STA $14E0,x

			JSL $019138

			PLA
			STA $14E0,x
			PLA
			STA $E4,x

			LDA $1588,x
			AND #$03
			BEQ RETURN1
                    
			JSR SUB_9A04

			LDA #$18		;shake ground
                    		STA $1887   

			;LDA $77
                    		;AND #$04
                    		;BEQ NO_LOCK_MARIO
                    		;LDA #$18      ; set timer to freeze mario
                    		;STA $18BD

;NO_LOCK_MARIO                    
                    		LDA #$09		;play sound effect
                    		STA $1DFC

			LDA #$40
			STA $1540,x

			INC $1534,x
RETURN1			RTS                     

;-----------------------------------------------------------------------------------------
; state 2
;-----------------------------------------------------------------------------------------

RETURNING		LDA $1540,x
			BNE RETURN2

			STZ $1528,x

			LDA $E4,x
			CMP $151C,x
			BNE RISE

			STZ $1534,x
			RTS

RISE			LDY $C2,x
			LDA RETURN_SPEED,y
			STA $B6,x
			JSL $01802A
RETURN2			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_GFX			JSR GET_DRAW_INFO

			LDA $1528,x
			STA $02
			PHX
			LDX #$03
			CMP #$00
			BEQ LOOP_START
			INX
LOOP_START		LDA $00
			CLC
			ADC X_OFFSET,x
			STA $0300,y

			LDA $01
			CLC
			ADC Y_OFFSET,x
			STA $0301,y

			LDA PROPERTIES,x
			ORA $64
			STA $0303,y

			LDA TILE_MAP,x
			CPX #$04
			BNE NORMAL_TILE
			PHX
			LDX $02
			CPX #$02
			BNE NOT_ANGRY
			LDA #ANGRY_TILE
NOT_ANGRY		PLX
NORMAL_TILE		STA $0302,y

			INY
			INY
			INY
			INY
			DEX
			BPL LOOP_START

			PLX

			LDY #$02
			LDA #$04
			JSL $01B7B3
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; speed related
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_9A04		LDA $1588,x
			BMI THWOMP_1
			LDA #$00
			LDY $15B8,x
			BEQ THWOMP_2
THWOMP_1		LDA #$18
THWOMP_2		STA $B6,x
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00
			LDA $E4,x
			STA $00
			LDA $14E0,x
			STA $01
			REP #$20
			LDA $94
			SEC
			SBC $00
			STA $0E
			BPL SPR_L16
			INY
SPR_L16			SEP #$20
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1			dcb $0C,$1C
SPR_T2			dcb $01,$02

GET_DRAW_INFO		STZ $186C,x
			STZ $15A0,x
			LDA $E4,x
			CMP $1A
			LDA $14E0,x
			SBC $1B
			BEQ ON_SCREEN_X
			INC $15A0,x

ON_SCREEN_X		LDA $14E0,x
			XBA
			LDA $E4,x
			REP #$20
			SEC
			SBC $1A
			CLC
			ADC.W #$0040
			CMP.W #$0180
			SEP #$20
			ROL A
			AND #$01
			STA $15C4,x
			BNE INVALID

			LDY #$00
			LDA $1662,x
			AND #$20
			BEQ ON_SCREEN_LOOP
			INY
ON_SCREEN_LOOP		LDA $D8,x
			CLC
			ADC SPR_T1,y
			PHP
 			CMP $1C
			ROL $00
			PLP
			LDA $14D4,x
			ADC #$00
			LSR $00
			SBC $1D
			BEQ ON_SCREEN_Y
			LDA $186C,x
			ORA SPR_T2,y
			STA $186C,x
ON_SCREEN_Y		DEY
			BPL ON_SCREEN_LOOP

			LDY $15EA,x
			LDA $E4,x
			SEC
			SBC $1A
			STA $00
			LDA $D8,x
			SEC
			SBC $1C
			STA $01
			RTS

INVALID			PLA
			PLA
			RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12			dcb $40,$B0
SPR_T13			dcb $01,$FF
SPR_T14			dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90
			dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0
SPR_T15			dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF
			dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X1	LDA #$02
			BRA STORE_03
SUB_OFF_SCREEN_X2  	LDA #$04
			BRA STORE_03
SUB_OFF_SCREEN_X3	LDA #$06
			BRA STORE_03
SUB_OFF_SCREEN_X4	LDA #$08
			BRA STORE_03
SUB_OFF_SCREEN_X5	LDA #$0A
			BRA STORE_03
SUB_OFF_SCREEN_X6	LDA #$0C
			BRA STORE_03
SUB_OFF_SCREEN_X7	LDA #$0E
STORE_03		STA $03
			BRA START_SUB
SUB_OFF_SCREEN_X0	STZ $03

START_SUB		JSR SUB_IS_OFF_SCREEN
			BEQ RETURN_35
			LDA $5B
			AND #$01
			BNE VERTICAL_LEVEL
			LDA $D8,x
			CLC
			ADC #$50
			LDA $14D4,x
			ADC #$00
			CMP #$02
			BPL ERASE_SPRITE
			LDA $167A,x
			AND #$04
			BNE RETURN_35
			LDA $13
			AND #$01
			ORA $03
			STA $01
			TAY
  			LDA $1A
			CLC
			ADC SPR_T14,y
			ROL $00
			CMP $E4,x
			PHP
			LDA $1B
			LSR $00
			ADC SPR_T15,y
			PLP
			SBC $14E0,x
			STA $00
			LSR $01
			BCC SPR_L31
			EOR #$80
			STA $00
SPR_L31			LDA $00
			BPL RETURN_35
ERASE_SPRITE		LDA $14C8,x
			CMP #$08
			BCC KILL_SPRITE
			LDY $161A,x
			CPY #$FF
			BEQ KILL_SPRITE
			LDA #$00
			STA $1938,y
KILL_SPRITE		STZ $14C8,x
RETURN_35		RTS

VERTICAL_LEVEL		LDA $167A,x
			AND #$04
			BNE RETURN_35
			LDA $13
			LSR A
			BCS RETURN_35
			LDA $E4,x
			CMP #$00
			LDA $14E0,x
			SBC #$00
			CMP #$02
			BCS ERASE_SPRITE
			LDA $13
			LSR A
			AND #$01
			STA $01
			TAY
			LDA $1C
			CLC
			ADC SPR_T12,y
			ROL $00
			CMP $D8,x
			PHP
			LDA.W $001D
			LSR $00
			ADC SPR_T13,y
			PLP
			SBC $14D4,x
			STA $00
			LDY $01
			BEQ SPR_L38
			EOR #$80
  			STA $00
SPR_L38 		LDA $00
			BPL RETURN_35
			BMI ERASE_SPRITE

SUB_IS_OFF_SCREEN	LDA $15A0,x
			ORA $186C,x
			RTS